const logger = require('structure-logger')

module.exports = function pluginsInterface(props = {}) {
  if(!props.plugins) props.plugins = {}

  let list        = props.list || []
  let migrations  = []
  let socketHandlers  = []
  let pluginsList = []
  let routes      = []

  // Build out the list of plugins and their resources & migrations
  for(let i = 0, l = list.length; i < l; i++) {
    const item = list[i]
    let plugin = null

    try {

      // Path to a plugin or a node module
      if(typeof item === 'string') {
        plugin = require(item)
      } else {
        plugin = item
      }

      // If this is route middleware
      if(plugin.settings && plugin.settings.pluginName) {

        props.plugins[plugin.settings.pluginName] = {
          resources: plugin.resources,
          settings: plugin.settings
        }

      }

      if(plugin.settings && plugin.settings.migrations && plugin.settings.migrations.tables) {
        migrations = migrations.concat(plugin.settings.migrations.tables)
      }

      if(plugin.settings && plugin.settings.socketHandlers) {
        for (const eventName in plugin.settings.socketHandlers) {
          socketHandlers.push({
            eventName,
            handler: plugin.settings.socketHandlers[eventName]
          })
        }
      }

      pluginsList.push(plugin)
    } catch(e) {
      logger.error('Could not load plugin', (typeof item === 'string') ? item : '')
      logger.error(e)

      continue
    }
  }

  // Provide all plugins with copy of props
  for(let i = 0, l = pluginsList.length; i < l; i++) {
    const propsCopy = Object.assign({}, props)
    const plugin = pluginsList[i]

    const pluginRoutes = (plugin.default) ? plugin.default(propsCopy) : plugin(propsCopy)

    // Not all plugins have routes
    if (pluginRoutes) {

      // If a plugin is providing multiple routes, if not transform into an array
      let routesArray = (pluginRoutes instanceof Array) ? pluginRoutes : new Array(pluginRoutes)

      for(let j = 0, k = routesArray.length; j < k; j++) {

        const routeItem = routesArray[j]

        routes.push({
          routeName: routeItem.routeName,
          routes: routeItem.routes
        })
      }

    }

  }

  return {
    migrations,
    routes,
    socketHandlers
  }

}
