const path = require('path')
const pluginsInterface = require('../../index')
const Plugin1 = require('../helpers/plugin1')
const Plugin2 = require('../helpers/plugin2')
const Plugin3 = require('../helpers/plugin3')
const Plugin4 = require('../helpers/plugin4')
const Plugin5 = require('../helpers/plugin5')
const Plugin6 = require('../helpers/plugin6')

class Controller {}
class Model {}

describe('Plugin', function() {

  it('should load a plugin by string', function() {

    const plugins = pluginsInterface({
      Controller,
      Model,
      list: [
        path.join(__dirname, '../helpers/plugin2'),
        path.join(__dirname, '../helpers/plugin1')
      ]
    })

    expect(plugins.routes.length).to.equal(2)
    expect(plugins.routes[0].routeName).to.equal('plugin2')
    expect(plugins.routes[0].routes.stack[0].route.path).to.equal('/:id2')
    expect(plugins.routes[1].routeName).to.equal('plugin1')
    expect(plugins.routes[1].routes.stack[0].route.path).to.equal('/:id1')

  })

  it('should load a plugin by object', function() {

    const plugins = pluginsInterface({
      Controller,
      Model,
      list: [
        Plugin2,
        Plugin1
      ]
    })

    expect(plugins.routes.length).to.equal(2)
    expect(plugins.routes[0].routeName).to.equal('plugin2')
    expect(plugins.routes[0].routes.stack.length).to.equal(1)
    expect(plugins.routes[0].routes.stack[0].route.path).to.equal('/:id2')
    expect(plugins.routes[1].routeName).to.equal('plugin1')
    expect(plugins.routes[1].routes.stack.length).to.equal(1)
    expect(plugins.routes[1].routes.stack[0].route.path).to.equal('/:id1')

  })

  it('should load a plugin with an array of routes', function() {

    const plugins = pluginsInterface({
      Controller,
      Model,
      list: [
        Plugin3
      ]
    })

    expect(plugins.routes.length).to.equal(2)
    expect(plugins.routes[0].routeName).to.equal('plugin3a')
    expect(plugins.routes[0].routes.stack.length).to.equal(1)
    expect(plugins.routes[0].routes.stack[0].route.path).to.equal('/:id3')
    expect(plugins.routes[1].routeName).to.equal('plugin3b')
    expect(plugins.routes[1].routes.stack.length).to.equal(1)
    expect(plugins.routes[1].routes.stack[0].route.path).to.equal('/:id3')

  })

  it('should load a plugin that defines no routes', function() {

    const plugins = pluginsInterface({
      Controller,
      Model,
      list: [
        Plugin4
      ]
    })

    expect(plugins.routes.length).to.equal(0)

  })

  it('should load plugins that share routerName', function() {

    const plugins = pluginsInterface({
      Controller,
      Model,
      list: [
        Plugin1,
        Plugin5
      ]
    })

    expect(plugins.routes.length).to.equal(2)
    expect(plugins.routes[0].routeName).to.equal('plugin1')
    expect(plugins.routes[0].routes.stack.length).to.equal(1)
    expect(plugins.routes[0].routes.stack[0].route.path).to.equal('/:id1')
    expect(plugins.routes[1].routeName).to.equal('plugin1')
    expect(plugins.routes[1].routes.stack.length).to.equal(1)
    expect(plugins.routes[1].routes.stack[0].route.path).to.equal('/:id5')

  })

  it('should load plugins with socket event handlers', function() {

    const plugins = pluginsInterface({
      Controller,
      Model,
      list: [
        Plugin6
      ]
    })

    expect(plugins.socketHandlers.length).to.equal(1)
    expect(plugins.socketHandlers[0].eventName).to.equal('socket-event')
    expect(plugins.socketHandlers[0].handler).to.be.a.function

  })

  it('should assemble table migrations', function() {

    const plugins = pluginsInterface({
      list: [
        'structure-users'
      ]
    })

    expect(plugins.migrations.length > 0)

  })

})
